from .base import Base
from datetime import datetime

class Categories(Base):
    """Class for Categories APIs."""

    def __init__(self):
        Base.__init__(self, attribute_type='CATEGORY')

    def sync(self, sync_after: datetime = None):
        """
        Syncs the latest API data to DB.
        """
        generator = self.get_all_generator(sync_after=sync_after)
        for items in generator:
            category_attributes = []
            for category in items['data']:
                if self.attribute_is_valid(category):
                    if category['sub_category'] and category['name'] != category['sub_category']:
                        category['name'] = '{0} / {1}'.format(category['name'], category['sub_category'])

                    category_attributes.append({
                        'attribute_type': self.attribute_type,
                        'display_name': self.attribute_type.replace('_', ' ').title(),
                        'value': category['name'],
                        'source_id': category['id'],
                        'active': category['is_enabled'],
                        'detail': None
                    })

            self.bulk_create_or_update_expense_attributes(category_attributes, True)



from datetime import datetime
from typing import Dict, List

from apps.workspaces.models import Workspace

from fyle_accounting_mappings.models import ExpenseAttribute


class Base:
    """The base class for all API classes."""

    def __init__(self, attribute_type: str = None, query_params: dict = {}):
        self.attribute_type = attribute_type
        self.query_params = query_params
        self.connection = None
        self.workspace_id = None
        self.workspace = None


    def set_connection(self, connection):
        self.connection = connection


    def set_workspace_id(self, workspace_id):
        self.workspace_id = workspace_id
        self.workspace = Workspace.objects.filter(id=self.workspace_id).first()


    def format_date(self, last_synced_at: datetime) -> str:
        """
        Formats the date in the format of gte.2021-09-30T11:00:57.000Z
        """
        return 'gte.{}'.format(datetime.strftime(last_synced_at, '%Y-%m-%dT%H:%M:%S.000Z'))


    def __get_last_synced_at(self):
        """
        Returns the last time the API was synced.
        """
        return ExpenseAttribute.get_last_synced_at(self.attribute_type, self.workspace_id)


    def construct_query_params(self, sync_after: datetime = None) -> dict:
        """
        Constructs the query params for the API call.
        :return: dict
        """
        if sync_after:
            updated_at = self.format_date(sync_after)
        else:
            last_synced_record = self.__get_last_synced_at()
            updated_at = self.format_date(last_synced_record.updated_at) if last_synced_record else None

        params = {'order': 'updated_at.desc'}
        params.update(self.query_params)

        if sync_after:
            params['updated_at'] = updated_at
        elif updated_at and self.attribute_type not in ('CATEGORY', 'EMPLOYEE', 'CORPORATE_CARD', 'MERCHANT'):
            params['updated_at'] = updated_at

        return params


    def get_all_generator(self, sync_after: datetime = None):
        """
        Returns the generator for retrieving data from the API.
        :return: Generator
        """
        query_params = self.construct_query_params(sync_after)

        return self.connection.list_all(query_params)

    def post_bulk(self, payload: List[Dict]):
        """
        Post data to Fyle in Bulk
        """
        return self.connection.post_bulk({'data': payload})
    
    def post(self, payload: Dict):
        """
        Post date to Fyle
        """
        return self.connection.post({'data': payload})

    def bulk_create_or_update_expense_attributes(self, attributes: List[dict], update_existing: bool = False) -> None:
        """
        Bulk creates or updates expense attributes.
        :param attributes: List of expense attributes.
        :param update_existing: If True, updates/creates the existing expense attributes.
        """
        ExpenseAttribute.bulk_create_or_update_expense_attributes(
            attributes, self.attribute_type, self.workspace_id, update_existing
        )

    def __construct_expense_attribute_objects(self, generator) -> List[dict]:
        """
        Constructs the expense attribute objects.
        :param generator: Generator
        :return: List of expense attribute objects.
        """
        attributes = []
        for items in generator:
            for row in items['data']:
                if self.attribute_is_valid(row):
                    attributes.append({
                        'attribute_type': self.attribute_type,
                        'display_name': self.attribute_type.replace('_', ' ').title(),
                        'value': row['name'],
                        'active': True,
                        'source_id': row['id']
                    })

        return attributes

    def attribute_is_valid(self, attribute):
        """
        Validate whether attribute is from the same org or not.
        """
        if hasattr(self.workspace, 'fyle_org_id'):
            return attribute['org_id'] == self.workspace.fyle_org_id

        elif hasattr(self.workspace, 'org_id'):
            return attribute['org_id'] == self.workspace.org_id


    def sync(self, sync_after: datetime = None) -> None:
        """
        Syncs the latest API data to DB.
        """
        generator = self.get_all_generator(sync_after)
        attributes = self.__construct_expense_attribute_objects(generator)
        self.bulk_create_or_update_expense_attributes(attributes)

    def get_by_id(self, id):
        """
        Get Single Resource object by ID
        :param id: Resource object ID
        :return: Resource Object
        """
        return self.connection.get_by_id(id)['data']



from .base import Base


class ExpenseCustomFields(Base):
    """Class for Expense Custom Fields APIs."""

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        query_params = {'order': 'updated_at.desc', 'is_custom': 'eq.true', 'type': 'eq.SELECT', 'is_enabled': 'eq.true'}
        generator = self.connection.list_all(query_params)

        for items in generator:
            for row in items['data']:
                if self.attribute_is_valid(row):
                    attributes = []
                    count = 1
                    attribute_type = row['field_name'].upper().replace(' ', '_')
                    for option in row['options']:
                        attributes.append({
                            'attribute_type': attribute_type,
                            'display_name': row['field_name'],
                            'value': option,
                            'active': True,
                            'source_id': 'expense_custom_field.{}.{}'.format(row['field_name'].lower(), count),
                            'detail': {
                                'custom_field_id': row['id'],
                                'placeholder': row['placeholder'],
                                'is_mandatory': row['is_mandatory'],
                                'is_dependent': False
                            }
                        })
                        count = count + 1
                    self.attribute_type = attribute_type
                    self.bulk_create_or_update_expense_attributes(attributes, True)

    def list_all(self, query_params=None):
        """
        List all the custom fields
        """
        if not query_params:
            query_params = {'order': 'updated_at.desc', 'is_custom': 'eq.true', 'is_enabled': 'eq.true'}
        generator = self.connection.list_all(query_params)
        custom_fields = []

        for items in generator:
            custom_fields = items['data']

        return custom_fields



from .base import Base

from apps.fyle.models import Reimbursement


class Reimbursements(Base):
    """Class for Reimbursements APIs."""

    def __construct_query_params(self) -> dict:
        """
        Constructs the query params for the API call.
        :return: dict
        """
        last_synced_record = Reimbursement.get_last_synced_at(self.workspace_id)
        updated_at = self.format_date(last_synced_record.updated_at) if last_synced_record else None

        query_params = {'order': 'updated_at.desc'}

        if updated_at:
            query_params['updated_at'] = updated_at

        return query_params


    def __get_all_generator(self):
        """
        Returns the generator for retrieving data from the API.
        :return: Generator
        """
        query_params = self.__construct_query_params()

        return self.connection.list_all(query_params)
    

    def search_reimbursements(self, query_params):
        """
        Get of reimbursements filtered on query parameters
        :return: Generator
        """
        query_params['order'] = 'updated_at.desc'
        return self.connection.list_all(query_params)

    def bulk_post_reimbursements(self, data):
        """
        Post of reimbursements
        """
        payload = {
            'data': data
        }
        return self.connection.bulk_post_reimbursements(payload)

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        generator = self.__get_all_generator()
        for items in generator:
            Reimbursement.create_or_update_reimbursement_objects(items['data'], self.workspace_id)
